from typing import Union

from .utils.to_int import to_int


def is_odd(input: Union[int, str]) -> bool:
    input = to_int(input)

    if input == None:
        return False

    return bool(input % 2)



import unittest

from pyvalidator import *


class TestIsMultiByte(unittest.TestCase):

    def test_valid_multibyte(self):
        self.assertTrue(is_multibyte('ひらがな・カタカナ、．漢字'))
        self.assertTrue(is_multibyte('あいうえお foobar'))
        self.assertTrue(is_multibyte('ｶﾀｶﾅ'))
        self.assertTrue(is_multibyte('中文'))
        self.assertTrue(is_multibyte('test＠example.com'))
        self.assertTrue(is_multibyte('1234abcDEｘｙｚ'))
        print('OK - test_valid_multibyte')

    def test_invalid_multibyte(self):
        self.assertFalse(is_multibyte('abc'))
        self.assertFalse(is_multibyte('abc123'))
        self.assertFalse(is_multibyte('<>@" *.'))
        print('OK - test_invalid_multibyte')



import unittest
import warnings

from pyvalidator import *


class TestIsOnline(unittest.TestCase):
	def valid_check(self, items = None):
		for item in items:
			try:
				self.assertTrue(is_online(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items = None):
		for item in items:
			try:
				self.assertFalse(is_online(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_is_online(self):
		try:
			self.valid_check([None]) # NOTE if internet connection on the machine is not available this test might fail
		except:
			warnings.warn('this test is expected to fail if the internet connection is not available')

	def test_reachable_urls(self):
		valid_items = [
			'google.com',
			'yahoo.com',
			'https://linkedin.com',
			'http://stackoverflow.com/'
        ]
		try:
			self.valid_check(valid_items)
			print('OK - test_reachable_urls')
		except:
			warnings.warn('This means that either the machine is offline or the testing urls are down, in which case the tesst fail is expected')

	def test_unreachable_url(self):
		invalid_items = [
			' ',
			'thisshouldneverexist.no.way.com',
			'asd'
        ]
		self.invalid_check(invalid_items)
		print('OK - test_unreachable_url')



import unittest

from pyvalidator import *


class TestIsMacAddress(unittest.TestCase):
	def valid_check(self, items, options = {}):
		for item in items:
			try:
				self.assertTrue(is_mac_address(item, options))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items, options = {}):
		for item in items:
			try:
				self.assertFalse(is_mac_address(item, options))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_valid_mac_address(self):
		valid = [
			'ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:ab',
			'01:AB:03:04:05:06',
			'A9 C5 D4 9F EB D3',
			'01 02 03 04 05 ab',
			'01-02-03-04-05-ab',
			'0102.0304.05ab',
			'ab:ab:ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:06:07:ab',
			'01:AB:03:04:05:06:07:08',
			'A9 C5 D4 9F EB D3 B6 65',
			'01 02 03 04 05 06 07 ab',
			'01-02-03-04-05-06-07-ab',
			'0102.0304.0506.07ab',
        ]

		self.valid_check(valid)
		print('OK - test_valid_mac_address')

	def test_invalid_mac_address(self):
		invalid = [
			' ',
			'0:0:0:0',
			' : ',
			'abc',
			'01:02:03:04:05',
			'01:02:03:04:05:z0',
			'01:02:03:04::ab',
			'1:2:3:4:5:6',
			'AB:CD:EF:GH:01:02',
			'A9C5 D4 9F EB D3',
			'01-02 03:04 05 ab',
			'0102.03:04.05ab',
			'900f/dffs/sdea',
			'01:02:03:04:05:06:07',
			'01:02:03:04:05:06:07:z0',
			'01:02:03:04:05:06::ab',
			'1:2:3:4:5:6:7:8',
			'AB:CD:EF:GH:01:02:03:04',
			'A9C5 D4 9F EB D3 B6 65',
			'01-02 03:04 05 06 07 ab',
			'0102.03:04.0506.07ab',
			'900f/dffs/sdea/54gh',
        ]
		self.invalid_check(invalid)
		print('OK - test_invalid_mac_address')

	def test_valid_mac_address_eui48(self):
		valid = [
			'ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:ab',
			'01:AB:03:04:05:06',
			'A9 C5 D4 9F EB D3',
			'01 02 03 04 05 ab',
			'01-02-03-04-05-ab',
			'0102.0304.05ab',
        ]

		self.valid_check(valid, { "eui": 48 })
		print('OK - test_valid_mac_address_eui48')

	def test_invalid_mac_address_eui48(self):
		invalid = [
			'ab:ab:ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:06:07:ab',
			'01:AB:03:04:05:06:07:08',
			'A9 C5 D4 9F EB D3 B6 65',
			'01 02 03 04 05 06 07 ab',
			'01-02-03-04-05-06-07-ab',
			'0102.0304.0506.07ab',
        ]
		self.invalid_check(invalid, { "eui": 48 })
		print('OK - test_invalid_mac_address_eui48')

	def test_valid_mac_address_eui64(self):
		valid = [
			'ab:ab:ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:06:07:ab',
			'01:AB:03:04:05:06:07:08',
			'A9 C5 D4 9F EB D3 B6 65',
			'01 02 03 04 05 06 07 ab',
			'01-02-03-04-05-06-07-ab',
			'0102.0304.0506.07ab',
        ]

		self.valid_check(valid, { "eui": 64 })
		print('OK - test_valid_mac_address_eui64')

	def test_invalid_mac_address_eui64(self):
		invalid = [
			'ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:ab',
			'01:AB:03:04:05:06',
			'A9 C5 D4 9F EB D3',
			'01 02 03 04 05 ab',
			'01-02-03-04-05-ab',
			'0102.0304.05ab',
        ]
		self.invalid_check(invalid, { "eui": 64 })
		print('OK - test_invalid_mac_address_eui64')

	def test_valid_mac_address_no_separator(self):
		valid = [
			'abababababab',
        	'FFFFFFFFFFFF',
			'0102030405ab',
			'01AB03040506',
			'abababababababab',
			'FFFFFFFFFFFFFFFF',
			'01020304050607ab',
			'01AB030405060708',
        ]

		self.valid_check(valid, { "no_separators": True })
		print('OK - test_valid_mac_address_no_separator')

	def test_invalid_mac_address_no_separators(self):
		invalid = [
			'abc',
			'01:02:03:04:05',
			'01:02:03:04::ab',
			'1:2:3:4:5:6',
			'AB:CD:EF:GH:01:02',
			'ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:ab',
			'01:AB:03:04:05:06',
			'0102030405',
			'01020304ab',
			'123456',
			'ABCDEFGH0102',
			'01:02:03:04:05:06:07',
			'01:02:03:04:05:06::ab',
			'1:2:3:4:5:6:7:8',
			'AB:CD:EF:GH:01:02:03:04',
			'ab:ab:ab:ab:ab:ab:ab:ab',
			'FF:FF:FF:FF:FF:FF:FF:FF',
			'01:02:03:04:05:06:07:ab',
			'01:AB:03:04:05:06:07:08',
			'01020304050607',
			'010203040506ab',
			'12345678',
			'ABCDEFGH01020304',
        ]
		self.invalid_check(invalid, { "no_separators": True })
		print('OK - test_invalid_mac_address_no_separators')

	def test_valid_mac_address_no_separator_eui48(self):
		valid = [
			'abababababab',
			'FFFFFFFFFFFF',
			'0102030405ab',
			'01AB03040506',
        ]

		self.valid_check(valid, { "no_separators": True, "eui": 48 })
		print('OK - test_valid_mac_address_no_separator_eui48')

	def test_invalid_mac_address_no_separators_eui48(self):
		invalid = [
			'abababababababab',
			'FFFFFFFFFFFFFFFF',
			'01020304050607ab',
			'01AB030405060708',
        ]
		self.invalid_check(invalid, { "no_separators": True, "eui": 48 })
		print('OK - test_invalid_mac_address_no_separators_eui48')

	def test_valid_mac_address_no_separator_eui64(self):
		valid = [
			'abababababababab',
			'FFFFFFFFFFFFFFFF',
			'01020304050607ab',
			'01AB030405060708',
        ]

		self.valid_check(valid, { "no_separators": True, "eui": 64 })
		print('OK - test_valid_mac_address_no_separator_eui64')

	def test_invalid_mac_address_no_separators_eui64(self):
		invalid = [
			'abababababab',
			'FFFFFFFFFFFF',
			'0102030405ab',
			'01AB03040506',
        ]
		self.invalid_check(invalid, { "no_separators": True, "eui": 64 })
		print('OK - test_invalid_mac_address_no_separators_eui64')



import unittest

from pyvalidator import *


class TestIsEven(unittest.TestCase):
	def valid_check(self, items):
		for item in items:
			try:
				self.assertTrue(is_even(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items):
		for item in items:
			try:
				self.assertFalse(is_even(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_valid_even(self):
		valid_items = [
			0,
			2,
			4,
			6,
			8,
			10,
			12,
			120,
			1000,
			-1000,
			'1000',
			'2',
			'120',
			'666',
			'-888'
        ]
		self.valid_check(valid_items)
		print('OK - test_valid_even')

	def test_invalid_even(self):
		invalid_items = [
			' ',
			'a',
			'.1',
			'1.0',
			3,
			5,
			111,
			-7,
			'17',
			'-21',
			'121'
        ]
		self.invalid_check(invalid_items)
		print('OK - test_invalid_even')



import unittest

from pyvalidator import *


class TestIsPrime(unittest.TestCase):
	def valid_check(self, items):
		for item in items:
			try:
				self.assertTrue(is_prime(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items):
		for item in items:
			try:
				self.assertFalse(is_prime(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_valid_prime(self):
		valid_items = [
			'2',
			'3',
			'5',
			'7',
			'11',
			'13',
			'17',
			'19',
			'23',
			'29',
			'31',
			2,
			3,
			5,
			7,
			11,
			13,
			17,
			19,
			23,
			29,
			31,
			37,
			41,
			43,
			47,
			53,
			59,
			61,
			67,
			71,
			73,
			79,
			83,
			89,
			97,
			21577
        ]
		self.valid_check(valid_items)
		print('OK - test_valid_prime')

	def test_invalid_prime(self):
		invalid_items = [
			1,
			4,
			6,
			8,
			9,
			10,
			12,
			14,
			15,
			16,
			18,
			20,
			21,
			22,
			24,
			25,
			26,
			27,
			28,
			30,
			32,
			33,
			34,
			35,
			36,
			38,
			39,
			40,
			42,
			44,
			45,
			46,
			48,
			49,
			50,
			51,
			52,
			54,
			55,
			56,
			57,
			58,
			60,
			62,
			63,
			'51',
			'52',
			'54',
			'55',
			'56',
			'57',
			'58',
			'60',
			'62',
			'63',
        ]
		self.invalid_check(invalid_items)
		print('OK - test_invalid_prime')



import inspect


def to_string(input):
    if type(input).__name__ == 'str':
        return input
    elif type(input).__name__ == 'int' or type(input).__name__ == 'float':
        return str(input)
    elif type(input).__name__ == 'dict' or type(input).__name__ == 'list' or inspect(input):
        return '[object Object]'
    return ''



import unittest

from pyvalidator import *


class TestIsUrl(unittest.TestCase):
	def valid_check(self, items, options = {}):
		for item in items:
			try:
				self.assertTrue(is_url(item, options))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items, options = {}):
		for item in items:
			try:
				self.assertFalse(is_url(item, options))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_valid_url(self):
		valid_items = [
            'https://stackoverflow.com/',
			'www.google.com',
            'HTTPS://stackoverflow.com/',
			'www.GOOGLE.com',
			'facebook.com',
			'facebook.COM',
			'http://rs.linkedin.com',
			'test.net',
			'unknowns.cc/yes/please',
			'worldofwarcraft.com/en-gb/game?races=blood-elf',
			'https://seeklogo.com/images/P/pearl-jam-alive-logo-8FA34991E4-seeklogo.com.png',
			'http://api.google.com/q?exp=a%7Cb',
			'https://companydomain.bamboohr.com/authorize.php?request=authorize&state=state&response_type=code&scope=openid&client_id=client_id&redirect_uri=redirecturl',
			'https://id.b2b.yahooinc.com/identity/oauth2/access_token?realm=true_story',
			'https://api.twitter.com/1.1/search/tweets.json',
			'https://api.twitter.com/1.1/search/%20/tweets.json',
			'http://foo.com/blahblah_(wikipedia)',
			'https://test-domain.com/',
			'ha.com',
			'www.go.abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzaaaaaaaaaaa'
        ]
		self.valid_check(valid_items)
		print('OK - test_valid_url')

	def test_invalid_url(self):
		invalid_items = [
            '',
            '.',
            'helloworld',
			'127.0.0.1',
			'ffs://silvermoon.com',
			'ffs:/silvermoon.com',
			'/some/random/path',
			'C:\\Documents',
			's.s',
			'http://mw1.google.com/mw-earth-vectordb/kml-samples/gp/seattle/gigapxl/$[level]/r$[y]_c$[x].jpg',
			'http://api.google.com/q?exp=a|b',
			'https://id.b2b.yahooinc.com/identity/oauth2/access_token?realm=true story',
			'https://api.twitter.com/1.1/search/ /tweets.json',
			'http://-test-domain.com',
			'http://test-domain-.com',
			'http://test_domain.com',
			'http://test-domain_.com',
			'http://_test-domain.com',
			'http://test*domain.com',
			'http://test!domain.com',
			'http://test-domain!.com',
			'http://test-domain*.com',
			'hello+world.io',
			'hello=world.io',
			'hello@world.io',
			'hello%world.io',
			'hello:world.io',
			'hello~world.io',
			'h.com',
			'www.go.abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzaaaaaaaaaaaa'
        ]
		self.invalid_check(invalid_items)
		print('OK - test_invalid_url')

	def test_valid_url_with_target_domains(self):
		valid_items = [
            'https://stackoverflow.com/',
			'www.google.com',
			'http://api.google.com/q?exp=a%7Cb',
        ]
		self.valid_check(valid_items, { "domains": [ "stackoverflow", "google" ] })
		print('OK - test_valid_url_with_target_domains')

	def test_valid_url_with_target_domain(self):
		valid_items = [
            'https://stackoverflow.com/'
        ]
		self.valid_check(valid_items, { "domains": [ "stackoverflow" ] })
		print('OK - test_valid_url_with_target_domain')

	def test_invalid_url_with_target_domains(self):
		invalid_items = [
            'https://stackoverflow.com/',
			'www.google.com',
			'http://api.google.com/q?exp=a%7Cb',
        ]
		self.invalid_check(invalid_items, { "domains": [ "facebook", "instagram" ] })
		print('OK - test_invalid_url_with_target_domains')

	def test_invalid_url_with_target_domain(self):
		invalid_items = [
            'https://stackoverflow.com/',
			'www.google.com',
			'http://api.google.com/q?exp=a%7Cb'
        ]
		self.invalid_check(invalid_items, { "domains": [ "twitter" ] })
		print('OK - test_invalid_url_with_target_domain')

	def test_valid_url_with_top_level_domains(self):
		items = [
            'https://stackoverflow.com/',
			'www.google.com',
			'en.wikipedia.org'
        ]
		self.valid_check(items, { "top_level_domains": [ "com", "org" ] })
		print('OK - test_valid_url_with_top_level_domains')

	def test_invalid_url_with_top_level_domains(self):
		items = [
            'https://stackoverflow.com/',
			'www.google.com'
        ]
		self.invalid_check(items, { "top_level_domains": [ "net" ] })
		print('OK - test_invalid_url_with_top_level_domains')

	def test_invalid_case_sensitive_url(self):
		items = [
            'HTTPS://stackoverflow.com/',
			'www.google.COM',
			'tWITTer.com'
        ]
		self.invalid_check(items, { "insensitive": False })
		print('OK - test_invalid_case_sensitive_url')

	def test_valid_url_with_no_scheme(self):
		items = [
            'en.wikipedia.org/',
			'www.youtube.com'
        ]
		self.valid_check(items, { "no_scheme": True })
		print('OK - test_valid_url_with_no_scheme')

	def test_invalid_url_with_no_scheme(self):
		items = [
            'https://en.wikipedia.org/',
			'http://www.youtube.com'
        ]
		self.invalid_check(items, { "no_scheme": True })
		print('OK - test_invalid_url_with_no_scheme')

	def test_valid_url_with_no_path(self):
		items = [
            'https://en.wikipedia.org/',
			'https://www.youtube.com'
        ]
		self.valid_check(items, { "with_no_path": True })
		print('OK - test_valid_url_with_no_path')

	def test_invalid_url_for_including_a_path(self):
		items = [
            'https://en.wikipedia.org/wiki/Grunge',
			'https://www.youtube.com/watch?v=iXjMbdHa-IU'
        ]
		self.invalid_check(items, { "with_no_path": True })
		print('OK - test_invalid_case_sensitive_url')



import unittest

from pyvalidator import *


class TestIsHtmlElement(unittest.TestCase):

    def test_valid_html_element(self):
        self.assertTrue(is_html_element('<div></div>'))
        self.assertTrue(is_html_element('<div ></ div>'))
        self.assertTrue(is_html_element('<    div ></ div>'))
        self.assertTrue(is_html_element('<div class="container"></div>'))
        self.assertTrue(is_html_element('<a></a>'))
        self.assertTrue(is_html_element('<a href=""></a>'))
        self.assertTrue(is_html_element('<input type="password" />'))
        self.assertTrue(is_html_element('<input disabled />'))
        self.assertTrue(is_html_element('<!DOCTYPE>'))
        self.assertTrue(is_html_element('<h1></h1>'))
        self.assertTrue(is_html_element('<h2></h2>'))
        self.assertTrue(is_html_element('<h3></h3>'))
        self.assertTrue(is_html_element('<h4></h4>'))
        self.assertTrue(is_html_element('<h5></h5>'))
        self.assertTrue(is_html_element('<h6>text</h6>'))
        self.assertTrue(is_html_element('<body><div></div></body>'))
        self.assertTrue(is_html_element('<base href="https://www.w3schools.com/" target="_blank">'))
        self.assertTrue(is_html_element('<img />'))
        self.assertTrue(is_html_element('<head></head>'))
        self.assertTrue(is_html_element('<label for="cars">Choose a car:</label>'))
        self.assertTrue(is_html_element('<p>&amp;</p>'))
        self.assertTrue(is_html_element('<textarea id="w3review" name="w3review" rows="4" cols="50"></textarea>'))
        self.assertTrue(is_html_element("""
            <style>
                h1 {color:red;}
                p {color:blue;}
            </style>
        """))
        self.assertTrue(is_html_element("""
            <!DOCTYPE html>
            <html>
                <head>
                    <title>Page Title</title>
                </head>
                <body>
                    <h1>This is a Heading</h1>
                    <p>This is a paragraph.</p>
                </body>
            </html>
        """))
        print('OK - test_valid_html_element')

    def test_invalid_html_element(self):
        self.assertFalse(is_html_element('<></>'))
        self.assertFalse(is_html_element('<<>></>'))
        self.assertFalse(is_html_element('<>'))
        self.assertFalse(is_html_element('<random></random>'))
        self.assertFalse(is_html_element('<body><body>'))
        self.assertFalse(is_html_element('0'))
        self.assertFalse(is_html_element('<h2></h1>'))
        self.assertFalse(is_html_element('<h3></h1>'))
        self.assertFalse(is_html_element('<h4></h1>'))
        self.assertFalse(is_html_element('<h5></h1>'))
        self.assertFalse(is_html_element('<h6>'))
        self.assertFalse(is_html_element('</h6>'))
        self.assertFalse(is_html_element('<h6></h6><h3></h3>'))
        self.assertFalse(is_html_element('<head>'))
        self.assertFalse(is_html_element('<img'))
        self.assertFalse(is_html_element('<img <>'))
        self.assertFalse(is_html_element('<<img <>'))
        self.assertFalse(is_html_element('<image />'))
        self.assertFalse(is_html_element('<image />'))
        self.assertFalse(is_html_element('<input></input>'))
        self.assertFalse(is_html_element('</textarea>'))
        self.assertFalse(is_html_element('</span>'))
        self.assertFalse(is_html_element('</span>'))
        self.assertFalse(is_html_element('<html lang="en">'))
        self.assertFalse(is_html_element('.'))
        self.assertFalse(is_html_element(''))
        self.assertFalse(is_html_element(' '))
        self.assertFalse(is_html_element('<p>paragraph</p> some text after'))
        self.assertFalse(is_html_element('some text before <p>paragraph</p>'))
        self.assertFalse(is_html_element("""
            <style>
                h1 {color:red;}
                p {color:blue;}
            <style>
        """))
        print('OK - test_invalid_html_element')

    def test_valid_html_element_contains(self):
        self.assertTrue(is_html_element('some text <p>paragraph</p>, exactly.', { "contains": True }))
        self.assertTrue(is_html_element('some text <br>brake, exactly.', { "contains": True }))
        self.assertTrue(is_html_element('some text <input /> input, exactly.', { "contains": True }))
        print('OK - test_valid_html_element_contains')



import unittest

from pyvalidator import *


class TestIsOdd(unittest.TestCase):
	def valid_check(self, items):
		for item in items:
			try:
				self.assertTrue(is_odd(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def invalid_check(self, items):
		for item in items:
			try:
				self.assertFalse(is_odd(item))
			except Exception as e:
				print(f'failed for input: {item}')
				raise e

	def test_valid_odd(self):
		valid_items = [
			1,
			3,
			5,
			7,
			9,
			111,
			2019,
			-1
        ]
		self.valid_check(valid_items)
		print('OK - test_valid_odd')

	def test_invalid_odd(self):
		invalid_items = [
			' ',
			'a',
			'.1',
			'1.1',
			0,
			2,
			110,
			220,
			2020,
			'2020',
			'666',
			'2',
			'0'
        ]
		self.invalid_check(invalid_items)
		print('OK - test_invalid_odd')
